import React, {Component} from 'react';
import ReactDom from 'react-dom';
import styled from 'styled-components';
import {Input, IInputProps} from '../Input/Input';

const StyledSuggestItem = styled.div`
  font-family: ${({theme}) => theme.font.family};
  font-size: 16px;
  padding: ${({theme}) => `${theme.padding.item} ${theme.padding.element}`};
  cursor: pointer;

  :hover {
    background-color: rgba(201,210,255,0.23);
  }
`;

const SuggestItem = (props) => {
  return (
    <StyledSuggestItem {...props}>
      {props.value}
    </StyledSuggestItem>
  );
};

const SuggestList = styled.div`
  position: relative;
  display: ${({visible}) => visible ? 'flex' : 'none'};
  flex-direction: column;
  width: 100%;
  color: ${({theme}) => theme.color.text};
  min-height: ${({theme}) => theme.font.element};
  max-height: 152px;
  box-sizing: border-box;
  overflow: scroll;
`;

const Container = styled.div`
  width: 100%;
  border-radius: 4px;
  box-shadow: ${({visible}) => visible ? '0 5px 10px 0 #d4d4d4' : '0'};
  background-color: ${({theme}) => theme.color.background};
`;

export interface IAutocompleteProps extends IInputProps {
  onSelect: (value: string) => void;
  list: string[];
  defaultValue?: any;
}

export class Autocomplete extends Component <IAutocompleteProps, any> {
  public static defaultProps = {
    onChange: () => null,
    onBlur: () => null,
    onFocus: () => null,
    onSelect: () => null,
  };

  public state = {
    value: '',
    visible: Boolean(this.props.list.length) || Boolean(this.props.defaultValue),
    prevVisible: Boolean(this.props.list.length) || Boolean(this.props.defaultValue),
  };

  private container: React.RefObject<any> = React.createRef();

  public componentDidMount() {
    document.addEventListener('click', this.onDocumentClick);
  }

  public componentWillUnmount() {
    document.removeEventListener('click', this.onDocumentClick);
  }

  public static getDerivedStateFromProps(props, state) {
    const {list} = props;
    const visible = Boolean(list.length);

    if (visible !== state.prevVisible) {
      return {
        visible,
        prevVisible: visible,
      };
    }

    return null;
  }

  private onSelect = (item) => {
    this.setState({
      value: item,
      visible: false,
    });

    this.props.onSelect(item);
  };

  private onDocumentClick = (event) => {
    if (this.state.visible && !ReactDom.findDOMNode(this).contains(event.target)) {
      this.setState({
        visible: false,
      });
    }
  };

  public render() {
    const {list, name, placeholder, onSelect, joined, ...rest} = this.props;
    const {value, visible} = this.state;

    return (
      <Container ref={this.container} visible={visible}>
        <Input
          {...rest}
          name={name}
          flat={visible}
          joined={joined}
          value={value}
          placeholder={placeholder}
        />
        <SuggestList visible={visible}>
          {list.map((item, index) => (
            <SuggestItem
              key={index}
              value={item}
              onClick={this.onSelect.bind(this, item)}
            />
          ))}
        </SuggestList>
      </Container>
    );
  }
}
