import React, {Component, ComponentClass, InputHTMLAttributes} from 'react';
import ReactDOM from 'react-dom';
import styled from 'styled-components';
import moment, {Moment} from 'moment';
import 'moment/locale/ru';
import {DayPicker} from './DayPicker';
import {MonthPicker} from './MonthPicker';
import {YearPicker} from './YearPicker';

moment.locale('ru');

const Wrapper = styled.div.attrs({tabIndex: 1})`
  display: flex;
  flex-direction: column;
  justify-content: space-between;
  font-family: Tahoma,sans-serif;
  outline: none;
  position: relative;
`;

const PickerWrapper = styled.div`
  position: absolute;
  top: 100%;
  left: 0;
  z-index: 1;
  background-color: #ffffff;
`;

interface IRefs {
  wrapper;
}

interface IDatePickerProps {
  value: string;
  format: string;
  displayFormat: string;
  name: string;
  onChange: (value: string, name: string) => void;
  time: string;
  months: boolean;
  input: ComponentClass<InputHTMLAttributes<any>, any>;
}

interface IDatePickerState {
  displayValue: string;
  prevValue: string;
  date: Moment;
  opened: boolean;
  daysOpened: boolean;
  monthsOpened: boolean;
  yearsOpened: boolean;
  yearOfSelected: number;
}

export class DatePicker extends Component<IDatePickerProps, IDatePickerState> implements IRefs {

  public static defaultProps = {
    onChange: (..._: any) => null,
    format: 'DD, MMMM YYYY',
  };

  constructor(props, ...args) {
    super(props, ...args);

    const {value, displayFormat, format, months} = props;
    const date = moment(value, format);
    const displayValue = date.isValid() ? date.format(displayFormat || format) : '';

    this.state = {
      date: moment(),
      opened: false,
      displayValue,
      prevValue: value,
      daysOpened: !months,
      monthsOpened: months,
      yearsOpened: false,
      yearOfSelected: 0,
    };
  }

  public static getDerivedStateFromProps(props, state) {
    const {value, displayFormat, format} = props;
    const date = moment(value, format);
    const displayValue = date.isValid() ? date.format(displayFormat || format) : '';

    if (value !== state.prevValue) {
      return {
        displayValue,
        prevValue: value,
      };
    }

    return null;
  }

  public wrapper = React.createRef();

  private closeDatePicker = (event) => {
    event.preventDefault();

    const {current} = this.wrapper;
    const {relatedTarget} = event;

    if (ReactDOM.findDOMNode(current).contains(relatedTarget)) {
      return;
    }

    this.setState(() => ({
      opened: false,
    }));
  };

  private openDatePicker = (event) => {
    event.preventDefault();

    const {opened} = this.state;

    if (opened) {
      return;
    }

    this.setState(() => ({
      opened: true,
    }));
  };

  private onChangeHandler = (event) => {
    const value = event.target ? event.target.value : event;

    this.setState(() => ({
      displayValue: value,
    }));
  };

  private setDay = (date) => {
    this.setState(() => {
      const {format, displayFormat, name, onChange, time} = this.props;
      const value = date.format(format);
      const displayValue = date.format(displayFormat || format);

      onChange(value, name);

      return {
        date,
        displayValue,
        opened: Boolean(time),
      };
    });
  };

  private setMonth = (date, yearOfSelected) => {
    this.setState(() => {
      const {format, displayFormat, name, onChange} = this.props;
      const value = date.format(format);
      const displayValue = date.format(displayFormat || format);

      onChange(value, name);

      return {
        date,
        displayValue,
        opened: false,
        yearOfSelected,
      };
    });
  };

  private openMonths = (date) => {
    this.setState(() => {
      return {
        date,
        yearsOpened: false,
        monthsOpened: true,
      };
    });
  };

  private setYear = (date) => {
    this.setState(() => {
      return {
        date,
      };
    });
  };

  private clickAtYears = () => {
    this.setState({
      yearsOpened: true,
      monthsOpened: false,
    });
  };

  private renderDayPicker = () => {
    const { date } = this.state;
    const { time } = this.props;

    return (
      <DayPicker
        onSelect={this.setDay}
        date={date}
        time={time}
      />
    );
  };

  private renderYearPicker = () => {
    const { date, displayValue } = this.state;

    return (
      <YearPicker
        onSelect={this.openMonths}
        setYear={this.setYear}
        date={date}
        hasDisplayValue={displayValue}
      />
    );
  };

  private renderMonthPicker = () => {
    const { date, displayValue, yearOfSelected } = this.state;

    return (
      <MonthPicker
        onSelect={this.setMonth}
        yearOfSelected={yearOfSelected}
        date={date}
        hasDisplayValue={displayValue}
        clickAtYears={this.clickAtYears}
      />
    );
  };

  public render() {
    const {displayValue, opened, daysOpened, monthsOpened, yearsOpened} = this.state;
    const {input: Input, ...rest} = this.props;

    let picker;
    if (monthsOpened) {
      picker = this.renderMonthPicker();
    }
    if (daysOpened) {
      picker = this.renderDayPicker();
    }
    if (yearsOpened) {
      picker = this.renderYearPicker();
    }

    return (
      <Wrapper
        ref={this.wrapper}
        onBlur={this.closeDatePicker}
        onFocus={this.openDatePicker}
        onClick={this.openDatePicker}
      >
        <Input
          {...rest}
          tabIndex={-1}
          onChange={this.onChangeHandler}
          value={displayValue}
        />
        <PickerWrapper>
        {opened ? picker : null}
        </PickerWrapper>
      </Wrapper>
    );
  }
}
