import React, {Component} from 'react';
import styled from 'styled-components';
import moment from 'moment';
import {PickerHead} from './PickerHead';
import {ChooseTimeBlock} from './ChooseTimeBlock';

const StyledPicker = styled.div`
  width: 252px;
  padding: 5px;
  margin: 0;
  font-size: 15px;
  border: 1px solid hsla(0, 0%, 0%, 0.25);
  border-radius: 4px;
  transition: all .1s;
  display: flex;
  flex-direction: column;
  justify-content: space-between;
  font-family: Tahoma,sans-serif;
  user-select: none;
`;

const PickerNav = styled.div`
  display: flex;
  justify-content: space-between;
  align-items: center;
  padding: 0 5px 5px;
  border-bottom: 1px solid hsla(0, 0%, 0%, 0.25);
`;

const Day = styled.div`
  width: 32px;
  height: 32px;
  display: flex;
  justify-content: center;
  align-items: center;
  cursor: pointer;
  color: hsla(0, 0%, 0%, ${(props) => props.active ? 1 : 0.25});
  &:hover {
    background-color: #F0F0F0;
    border-radius: 5px;
  }
  &:focus {
    background-color: #5cc4ef;
  }
`;

const Svg = styled.svg`
  width: 32px;
  height: 32px;
  cursor: pointer;
  &:hover {
    background-color: #F0F0F0;
    border-radius: 5px;
  }
`;

const CurrentDate = styled.div`
  text-transform: capitalize;
  height: 32px;
  padding: 0 10px 0;
  cursor: pointer;
  display: flex;
  justify-content: center;
  align-items: center;
  &:hover {
    background-color: #F0F0F0;
    border-radius: 5px;
  }
`;

const Body = styled.div`
  display: flex;
  flex-wrap: wrap;
  justify-content: space-between;
  border-bottom: ${({time}) => time ? '1px solid hsla(0, 0%, 0%, 0.25)' : 'none'};
`;

enum DayMode {
  PREVIOUS,
  CURRENT,
  NEXT,
}

export class DayPicker extends Component<any, any> {
  public state = {
    date: this.props.date,
    days: [],
  };

  public componentDidMount() {
    const {date} = this.state;
    const days = this.createDays(date.clone());

    this.setState(() => ({
      days,
    }));
  }

  private onDayClick = (event, item) => {
    const {mode, day} = item;

    switch (mode) {
      case DayMode.PREVIOUS:
        this.prevMonth(event);
        break;
      case DayMode.CURRENT:
        this.setDate(event, day);
        break;
      case DayMode.NEXT:
        this.nextMonth(event);
        break;
    }
  };

  private onHoursChangeHandler = (event) => {
    event.preventDefault();

    const {target} = event;
    const {date} = this.state;
    const {onSelect} = this.props;
    const newDate = date.set('hour', target.value);

    this.setState(() => ({
      date: newDate,
    }));

    onSelect(newDate);
  };

  private onMinutesChangeHandler = (event) => {
    event.preventDefault();

    const {target} = event;
    const {date} = this.state;
    const {onSelect} = this.props;
    const newDate = date.set('minute', target.value);

    this.setState(() => ({
      date: newDate,
    }));

    onSelect(newDate);
  };

  private setDate = (event, day) => {
    event.preventDefault();

    const {date} = this.state;
    const {onSelect} = this.props;

    onSelect(date.set('date', day));
  };

  private prevMonth = (event) => {
    event.preventDefault();

    const {date} = this.state;
    const month = date.month();
    const year = date.year();

    if (month === 0) {
      this.setNewDays(11, year - 1);
    } else {
      this.setNewDays(month - 1, year);
    }
  };

  private nextMonth = (event) => {
    event.preventDefault();

    const {date} = this.state;
    const month = date.month();
    const year = date.year();

    if (month === 11) {
      this.setNewDays(0, year + 1);
    } else {
      this.setNewDays(month + 1, year);
    }
  };

  public render() {
    const {date, days} = this.state;
    const {time} = this.props;

    return (
      <StyledPicker>
        <PickerNav>
          <Svg onMouseDown={this.prevMonth}>
            <path d="M 17,12 l -5,5 l 5,5"/>
          </Svg>
          <CurrentDate>{date.format('MMMM, YYYY')}</CurrentDate>
          <Svg onMouseDown={this.nextMonth}>
            <path d="M 14,12 l 5,5 l -5,5"/>
          </Svg>
        </PickerNav>
        <PickerHead/>
        <Body time={time}>
          {days.map((item, index) => {
            const {key, dataDay, active, day} = item;

            return (
              <Day
                key={key}
                data-day={dataDay}
                onMouseDown={(event) => this.onDayClick(event, item)}
                active={active}
              >
                {day}
              </Day>
            );
          })}
        </Body>
        {
          time ?
            <ChooseTimeBlock
              date={date}
              onHoursChangeHandler={this.onHoursChangeHandler}
              onMinutesChangeHandler={this.onMinutesChangeHandler}
            />
          :
            null
        }
      </StyledPicker>
    );
  }

  private setNewDays(month: any, year: number | any) {
    const {date} = this.state;
    const newDate = date.set('month', month).set('year', year);
    const days = this.createDays(newDate);

    this.setState(() => ({
      date: newDate,
      days,
    }));
  }

  private createDays(newDate) {
    const date = newDate || moment();

    const startDayOfWeek = this.realDayOfWeek(date.clone().startOf('month').day());
    const daysInPrevMonth = date.clone().set('month', date.clone().month() - 1).daysInMonth();
    const daysInPrevToShow = startDayOfWeek - 1;
    const endDayOfWeek = this.realDayOfWeek(date.endOf('month').day());
    const daysInNextToShow = 7 - endDayOfWeek;
    const days = [];
    let allAdded = false;
    let currentAdded = false;
    let prevAdded = false;
    let nextAdded = false;

    for (let i = 1; !allAdded; ++i) {
      if (!prevAdded) {
        if (startDayOfWeek === 1 || i === startDayOfWeek) {
          i = 0;
          prevAdded = true;
        } else {
          const day = daysInPrevMonth - (daysInPrevToShow - i);
          days.push(
            {
              key: `prev-${i}`,
              dataDay: true,
              active: false,
              day,
              mode: DayMode.PREVIOUS,
            });
        }
      }
      if (!currentAdded && prevAdded) {
        if (i === date.daysInMonth()) {
          i = 0;
          currentAdded = true;
        } else {
          days.push(
            {
              key: `curr-${i}`,
              dataDay: true,
              active: true,
              day: i + 1,
              mode: DayMode.CURRENT,
            });
        }
      }
      if (!nextAdded && currentAdded) {
        if (i === daysInNextToShow) {
          i = 0;
          nextAdded = true;
        } else {
          days.push(
            {
              key: `next-${i + 1}`,
              dataDay: true,
              active: false,
              day: i + 1,
              mode: DayMode.NEXT,
            });
        }
      }
      if (nextAdded && currentAdded && prevAdded) {
        allAdded = true;
      }
    }

    return days;
  }

  private realDayOfWeek(day) {
    return day || 7;
  }
}
