import React, {Component} from 'react';
import ReactDOM from 'react-dom';
import styled from 'styled-components';

export enum EDirection {
  TOP = 'top',
  RIGHT = 'right',
  BOTTOM = 'bottom',
  LEFT = 'left',
}

const getColor = (wanted) => {
  return ({direction}) => wanted === direction ? 'white' : 'transparent';
};

const getTranslateX = ({direction}) => {
  switch (direction) {
    case EDirection.RIGHT:
      return '0%';
    case EDirection.LEFT:
      return '-100%';
    case EDirection.TOP:
    case EDirection.BOTTOM:
      return '-50%';
  }
};

const getTranslateY = ({direction}) => {
  switch (direction) {
    case EDirection.RIGHT:
    case EDirection.LEFT:
      return '-50%';
    case EDirection.TOP:
      return '0%';
    case EDirection.BOTTOM:
      return '-100%';
  }
};

const Container = styled.div`
  display: inline-flex;
  width: ${({fluid}) => fluid ? '100%' : 'auto'}
`;

const Content = styled.div`
  display: ${({active}) => active ? 'inline-block' : 'none'};
  position: absolute;
  box-shadow: 0 20px 70px 0 rgba(55, 88, 189, 0.21);
  padding: 40px;
  margin: 0 18px;
  top: ${({top}) => top};
  left: ${({left}) => left};
  transform: translateX(${getTranslateX}) translateY(${getTranslateY});
  background-color: #ffffff;
  z-index: 10000;

  ::after {
    content: '';
    position: absolute;
    top: 50%;
    right: ${({direction}) => direction === 'right' ? '100%' : '-32px'};
    margin-top: -16px;
    border-width: 16px;
    border-style: solid;
    border-color: ${getColor('top')} ${getColor('right')} ${getColor('bottom')} ${getColor('left')};
  }
`;

const CloseButton = styled.button`
  position: absolute;
  right: 10px;
  top: 10px;
  width: 12px;
  height: 12px;
  cursor: pointer;
  border: none;
  background-color: transparent;
  outline: none;
  background-image: url('${(props) => props.icon}');
  background-repeat: no-repeat;
  padding: 0;
`;

interface ITooltipProps {
  content: () => string | string;
  active: boolean;
  closeIcon: string;
  onClose: () => void;
  fluid: boolean;
  delay: number;
  direction: EDirection;
}

const Inner = ({children, left, top, direction, active, onClose, closeIcon}) => {
  return ReactDOM.createPortal(
    <Content direction={direction} top={top} left={left} active={active}>
      {Boolean(onClose) ? <CloseButton icon={closeIcon} onClick={onClose}/> : null}
      {children}
    </Content>,
    document.body,
  );
};

export class Tooltip extends Component<ITooltipProps, any> {

  public state = {
    top: 0,
    left: 0,
  };

  private ref: any = null;

  private setRef = (ref) => {
    if (!ref) {
      return;
    }

    this.ref = ref;
  };

  public componentDidMount(): void {
    setTimeout(this.calculatePosition, 500);
  }

  public calculatePosition = () => {
    const {direction} = this.props;
    const {top, left, width, height} = this.ref.firstChild.getBoundingClientRect();

    // RIGHT is default
    let tooltipTop = top + height / 2 + 'px';
    let tooltipLeft = left + width + 'px';

    switch (direction) {
      case EDirection.LEFT:
        tooltipTop = top + height / 2 + 'px';
        tooltipLeft = left + 'px';
        break;
      case EDirection.TOP:
        tooltipTop = top + 'px';
        tooltipLeft = left + width / 2 + 'px';
        break;
      case EDirection.BOTTOM:
        tooltipTop = top + height + 'px';
        tooltipLeft = left + width / 2 + 'px';
        break;
      default:
        break;
    }

    this.setState(() => ({
      top: tooltipTop,
      left: tooltipLeft,
    }));
  };

  public render() {
    const {children, direction, content, active, fluid, closeIcon, onClose} = this.props;
    const {top, left} = this.state;

    return (
      <>
        <Container fluid={fluid} ref={this.setRef}>
          {children}
        </Container>
        <Inner
          top={top}
          left={left}
          closeIcon={closeIcon}
          active={active}
          onClose={onClose}
          direction={direction}
        >
          {typeof content === 'function' ? content() : content}
        </Inner>
      </>
    );
  }

  public static defaultProps = {
    onClose: () => null,
    direction: EDirection.RIGHT,
  };
}
