import React, {createContext, useContext, ComponentType, createElement} from 'react';

type IFromRegistryComponent<T> = ComponentType<T> & { fromRegistry: boolean; rawComponent?: ComponentType<T> };

class Registry {
  private map: { [key: string]: ComponentType } = {};

  constructor(private name: string) {
  }

  public register(name: string, component: ComponentType): ComponentType {
    this.map[name] = component;

    component.displayName = `${this.name}/${name}`;

    return component;
  }

  public registerAll(componentsMap: { [key: string]: ComponentType }) {
    Object.keys(componentsMap).forEach((key) => {
      this.register(key, componentsMap[key]);
    });
  }

  public setAll(componentsMap: { [key: string]: ComponentType }) {
    Object.keys(componentsMap).forEach((key) => {
      this.map[key] = componentsMap[key];
    });
  }

  public get<T>(name: string): ComponentType<T> {
    const component = this.map[name];

    if (!component) {
      console.warn(`component ${name} not found in registry`);
      return () => null;
    }

    return component as ComponentType<T>;
  }

  public getAll() {
    return this.map;
  }
}

export const createRegistry = (
  name: string,
  components?: { [key: string]: ComponentType },
  parent?: Registry,
): Registry => {
  const registry = new Registry(name);

  if (components) {
    registry.registerAll(components);
  }

  if (parent) {
    registry.setAll(parent.getAll());
  }

  return registry;
};

export interface IRegistryContext {
  registry?: Registry;
}

const RegistryContext = createContext<IRegistryContext>({
  registry: null,
});

export const RegistryProvider = ({registry, children}) => {
  return (
    <RegistryContext.Provider value={{registry}}>
      {children}
    </RegistryContext.Provider>
  );
};

export const registry = createRegistry('aiokit');

export const useRegistry = () => {
  const ctx = useContext(RegistryContext);
  return ctx.registry || registry;
};

export function fromRegistry<T>(name: string): IFromRegistryComponent<T> {
  const Wrapper: IFromRegistryComponent<T> = (props: T) => {
    const registry = useRegistry();
    const Component = registry.get<T>(name);
    return createElement<T>(Component, props);
  };

  Wrapper.displayName = `useRegistry().${name}`;
  Wrapper.fromRegistry = true;
  Wrapper.rawComponent = registry.get<T>(name);

  return Wrapper;
}
